/*
 * @author Walther Neuper, member of the ISAC-team, 
 * Copyright (c) 2004 by Walther Neuper 
 * created Oct 7, 2004, 5:51:50 PM
 * Institute for Softwaretechnology, Graz University of Technology, Austria.
 * 
 * Use is subject to license terms.
 */
package isac.util.formulae;

import junit.framework.TestCase;

/**
 * @author Walther Neuper Oct 7, 2004, 5:51:50 PM
 * @version 1.0
 */
public class TestCalcHead extends TestCase {

	private CalcHead ch = new CalcHead();

	protected void setUp() {
		//the complete CalcHead for mini-subpbl
		ch.setStatusString("correct");
		//----------------------the headline
		ch.setHead("solve (x + 1 = 2, x)");

		//----------------------the model
		CalcHeadItemList its;
		CalcHeadItem it1 = new CalcHeadItem(), it2 = new CalcHeadItem();

		it1.setText("equality (x+1=2)");
		it2.setText("solveFor x");
		its = new CalcHeadItemList();
		its.addItem(it1);
		its.addItem(it2);
		ch.setGiven(its);

		it1.setText("matches (?a = ?b) (x+1=2)");
		its = new CalcHeadItemList();
		its.addItem(it1);
		ch.setWhere(its);

		it1.setText("solutions L");
		its = new CalcHeadItemList();
		its.addItem(it1);
		ch.setFind(its);

		//----------------------
		ch.setBelongsTo("Pbl");

		//----------------------the specification
		Specification s = new Specification();
		CalcHeadSimpleID id;

		id = new CalcHeadSimpleID();
		id.setID("Test.thy");
		s.setTheory(id);

		CalcHeadCompoundID idl = new ProblemID();//CalcHeadCompoundID --> 
		idl.addString("sqroot-test");
		idl.addString("univariate");
		idl.addString("equation");
		idl.addString("test");
		s.setProblem(idl);

		idl = new CalcHeadCompoundID();
		idl.addString("Test");
		idl.addString("squ-equ-test-subpbl1");
		s.setMethod(idl);
		
		ch.setSpecification(s);
	}

	public void testUsesProblem() {
		System.out.println("---BEGIN isac.util.formulae.TestCalcHead");		

		ProblemID p = new ProblemID("[sqroot-test,univariate,equation,test]");
		assertTrue("pbl belongs to class defined by itself",ch.usesProblem(p));
		
		p = new ProblemID("[equation,test]");
		assertFalse("usual case: not member of class",ch.usesProblem(p));
		assertTrue("usual case: member of class",ch.usesProblemClass(p));
		
		p = new ProblemID("[]");
		assertTrue("[] is superClass for all pbls",ch.usesProblemClass(p));

		System.out.println("---END isac.util.formulae.TestCalcHead");		
	}

}