/*
 * Created on Aug 13, 2003
 */
package isac.bridge;

/**
 * The sml-kernel is running in its own thread. This
 * Class is used for setting up and starting the thread. 
 * The communication with the bridge is done via Input/Output streams
 * @author rgradisc
 */
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.security.AccessControlException;

public class SMLThread implements Runnable {

  private Process process;
  public boolean exited;

  protected String execution_commandos[] = new String[2];   

  private InputStream smlInput;
  private OutputStream smlOutput;
  private BridgeMain bridge;  

  public SMLThread(BridgeMain bridge, String kernelExec, String kernelArgs) {    
    execution_commandos[0] = kernelExec;
    execution_commandos[1] = kernelArgs;
    bridge.log(1, kernelExec);
    bridge.log(1, kernelArgs);

    this.bridge = bridge;    
	initProcess();
    exited = false;    
  }

  //---------------------------------------------------------------
  /**
  * returns the InputStream from the Kernel
  **/
  public synchronized InputStream getInputStream() {
    try {      
      smlInput = process.getInputStream();      
    } catch (java.lang.NullPointerException null_pointer_exception) {
      bridge.log(1, "NullpointerException" + null_pointer_exception);
      try {
        wait(20);
      } catch (InterruptedException e) {
        System.out.println("getInputStream: " + e); // DEBUG
      }
      smlInput = getInputStream(); // recurse until stream ready
    }
    return smlInput;
  }

  //---------------------------------------------------------------
  /**
  * returns the OutputStream from the Kernel
  *
  **/
  public synchronized OutputStream getOutputStream() {
    try {
      smlOutput = process.getOutputStream();
    } catch (java.lang.NullPointerException null_pointer_exception) {
      try {
        wait(20);
      } catch (InterruptedException e) {
        System.out.println("getOutputStream: " + e); // DEBUG
      }
      smlOutput = getOutputStream(); // recurse until stream ready
    }
    return smlOutput;
  }

  /**
   * Initialize the process
   * and execute the sml process with the execution commandos
   */
  public synchronized void initProcess() {    
    Runtime runtime = Runtime.getRuntime();    
    // start sml
    try {      
	    try {        
        process = runtime.exec(execution_commandos);
      } catch (AccessControlException  e) { 
      	bridge.log(1, "AccessControlExecption:\n"+e.toString());       
        e.printStackTrace();
      }      
    } catch (IOException e) {
      e.printStackTrace();
      exited = true;
      bridge.log(1, "SMLThread: could not execute process");            
    }    
  }

  //---------------------------------------------------------------
  /**
  * starts the ISAC - Kernel
  *
  **/
  public void run() {    
    // wait until sml has terminated    
    while (!exited) {
      try {
        process.waitFor();
      } catch (InterruptedException e) {
        bridge.log(1, "SMLThread INTERRUPTED");
        break;
      }
      exited = true;
	  process.destroy(); // Destroy process and I/O streams
    }
    try {
      smlInput.close();
      smlOutput.close();
    } catch (IOException e) {
      bridge.log(1, "Cannot close SML Streams");
      e.printStackTrace();
      
    }
    bridge.log(1, "SML Thread: Streams closed");    
    //process.destroy(); // Destroy process and I/O streams        
    //bridge.log(1, "SML Thread: Process returned: "+process.exitValue());
  }
}
