/****************************************************************** 
 * Copyright (c) 2003, Mario Hochreiter, Student of 
 * Software Engineering at FH-Hagenberg, Austria. 
 * Project member of the isac team at the Institute for 
 * Software-Technologie, Graz University of Technology, Austria. 
 * 
 * Use is subject to license terms.
 ******************************************************************/

package isac.gui;

import isac.util.InformationProcessor;
import isac.gui.browser.BrowserWindow;
import isac.gui.browser.ContentPanel;
import isac.gui.browser.example.ExampleBrowserPanel;
import isac.gui.browser.method.MethodBrowserPanel;
import isac.gui.browser.problem.ProblemBrowserPanel;
import isac.gui.util.*;
import isac.session.SDialog;
import isac.util.Formalization;
import isac.wsdialog.DGuide;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.rmi.RemoteException;
import java.util.Locale;

import javax.swing.*;

import org.apache.log4j.Logger;
import org.w3c.dom.Node;

/**
 * Main window application for the ISAC system. Responsible for showing the user
 * all requested information and for the interaction with the user.
 * 
 * @author Mario Hochreiter
 * @version 0.1
 *  
 */
public class WindowApplication extends JFrame implements ActionListener,
    ILoginListener {
  static Logger logger = Logger.getLogger(WindowApplication.class.getName());

  /** Local settings for the user */
  private Locale locale;

  /** Container for the internal frames, which will contain the worksheets */
  private JDesktopPane worksheetDesktopPane;

  private JInternalFrame contentFrame;

  private JInternalFrame tryMatchFrame;

  private JInternalFrame tryRefineFrame;

  private JInternalFrame activeFrame;

  /** login dialoge */
  private LoginScreen loginScreen;

  private CalcHeadPanel tryMatchPanel;

  private CalcHeadPanel tryRefinePanel;

  private ContentPanel contentPanel;

  /** Top container that contains all elements of this frame */
  private JSplitPane splitPane;

  /** The most recent active worksheet */
  private Worksheet lastActiveWS;

  /** Required to logon a user and to obtain the <code>WSDialog</code> */
  private SDialog sessionDialog;

  /** Unique sessionId for this WindowApplication */
  private String sessionId;

  private SDialog sdialog;

  // LK 04.08.30
  // Menu Buttons
  //left side
  private JButton newWorksheetButton;
  
  private JButton exampleBrowserButton;
  
  private JButton problemBrowserButton;

  private JButton methodBrowserButton;

  private JButton theoryBrowserButton;

  //right side
  private JButton nextStepButton;

  private JButton autocalcButton;

  private JButton tacticsButton;

  private ExampleBrowserPanel exampleBrowserPanel;

  private ProblemBrowserPanel problemBrowserPanel;

  private MethodBrowserPanel methodBrowserPanel;

  private BrowserWindow exampleBrowserWindow;

  private BrowserWindow problemBrowserWindow;

  private BrowserWindow methodBrowserWindow;

  private JInternalFrame exampleBrowserFrame;

  private JInternalFrame problemBrowserFrame;

  private JInternalFrame methodBrowserFrame;

  /**
   * Panel that displays the different "browseable" hierarchies (e.g. the
   * example Browser).
   */
  //private BrowserPanel browserPanel;
  public WindowApplication() {
    locale = Locale.getDefault();

    this.setSize(new Dimension(1024, 768));
    Point screenCenter = this.screenCenter();
    this.setLocation(screenCenter);

    Container contentPane = this.getContentPane();
    this.buildMenuBar();

    this.createWorksheetPane();
    this.createBrowserPanel();
    this.buildSplitPane();

    contentPane.add(splitPane);

    JFrame.setDefaultLookAndFeelDecorated(true);
    this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    this.hide();

    loginScreen = new LoginScreen();
    loginScreen.addListener(this);
    loginScreen.show();
  }

  /* LK 04.08.27 */
  /**
   * login mehtod caled from <code>LoginScreen</code> Initialize the remote
   * isac objects <code>ObjectManager</code> and <code>SessionDialog</code>.
   */
  private void login(String username, String password) {
    sessionId = InformationProcessor.login(username, password);
    if (sessionId != null) {
      sdialog = InformationProcessor.getSDialog();
      loginScreen.dispose();
      this.show();
      //openNewWorksheet(DialogGuide.STARTFROM_EMPTY, CalcHead.VIEWSTYLE_FULL);
    } else {
      loginScreen.showErrorMsg();
    }
  }

  /**
   * Create a new DialogGuide that can be connected to a new Worksheet
   */
  private DGuide newDialog() {
    String dialogId;
    DGuide dialog = null;
    try {
      dialogId = sdialog.openDGuide(sessionId);
      dialog = sdialog.getDGuide(dialogId);
    } catch (RemoteException e) {
      logger.error("newDialog", e.detail);
    }
    return dialog;
  }

  //---------- Load the different hierarchies---------------------------

  private Hierarchy loadExpHierarchy() {
    String dummy = InformationProcessor.loadHierarchy(sessionId, "exp");
    System.out.println("WN##### loadExpHierarchy, dummy=" + dummy);
    XMLExampleParser expParser = new XMLExampleParser(dummy);
    Node root = expParser.getRoot();
    Hierarchy hierarchy = expParser.createExampleNodesHierarchy(root);
    return hierarchy;
  }

  private Hierarchy loadPblHierarchy() {
    String dummy = InformationProcessor.loadHierarchy(sessionId, "pbl");
    XMLProblemParser pblParser = new XMLProblemParser(dummy);
    Node root = pblParser.getRoot();
    Hierarchy hierarchy = pblParser.createProblemNodesHierarchy(root);
    return hierarchy;
  }

  private Hierarchy loadMetHierarchy() {
    String dummy = InformationProcessor.loadHierarchy(sessionId, "met");
    XMLMethodParser metParser = new XMLMethodParser(dummy);
    Node root = metParser.getRoot();
    Hierarchy hierarchy = metParser.createMethodNodesHierarchy(root);
    return hierarchy;
  }

  private void buildMenuBar() {

    JMenuBar menuBar = new JMenuBar();

    //left side
    newWorksheetButton = new JButton("NEW");
    newWorksheetButton.setName("New");
    newWorksheetButton.addActionListener(this);
    exampleBrowserButton = new JButton("Examples");
    exampleBrowserButton.setName("Examples");
    exampleBrowserButton.addActionListener(this);
    problemBrowserButton = new JButton("Problems");
    problemBrowserButton.setName("Problems");
    problemBrowserButton.addActionListener(this);
    methodBrowserButton = new JButton("Methods");
    methodBrowserButton.setName("Methods");
    methodBrowserButton.addActionListener(this);
    theoryBrowserButton = new JButton("Theories");
    theoryBrowserButton.setName("Theories");
    theoryBrowserButton.addActionListener(this);

    
    //right side
    nextStepButton = new JButton("Next");
    nextStepButton.setName("Next");
    nextStepButton.addActionListener(this);
    autocalcButton = new JButton("Auto");
    autocalcButton.setName("Auto");
    autocalcButton.addActionListener(this);
    tacticsButton = new JButton("Tactics");
    tacticsButton.setName("Tactics");
    tacticsButton.addActionListener(this);

    //add left menu buttons
    menuBar.add(newWorksheetButton);
    menuBar.add(exampleBrowserButton);
    menuBar.add(problemBrowserButton);
    menuBar.add(methodBrowserButton);
    menuBar.add(theoryBrowserButton);
    //seperate
    menuBar.add(new JLabel("              "));
    menuBar.add(new JLabel("              "));
    menuBar.add(new JLabel("              "));
    menuBar.add(new JLabel("              "));
    menuBar.add(new JLabel("              "));
    menuBar.add(new JLabel("              "));
    //add right menu buttons
    menuBar.add(nextStepButton);
    menuBar.add(autocalcButton);
    menuBar.add(tacticsButton);

    this.setJMenuBar(menuBar);
  }

  private void createBrowserPanel() {
    exampleBrowserPanel = new ExampleBrowserPanel(this, loadExpHierarchy());
    exampleBrowserWindow = new BrowserWindow(exampleBrowserPanel);
    exampleBrowserFrame = createInternalFrame("Example Browser", false, true,
                                              false, false);
    exampleBrowserFrame.getContentPane().add(exampleBrowserWindow);
    //    this.addInternalFrame(exampleBrowserFrame);
    //    exampleBrowserFrame.pack();
    //    exampleBrowserFrame.hide();

    problemBrowserPanel = new ProblemBrowserPanel(this, loadPblHierarchy());
    problemBrowserWindow = new BrowserWindow(problemBrowserPanel);
    problemBrowserFrame = createInternalFrame("Problem Browser", false, true,
                                              false, false);
    problemBrowserFrame.getContentPane().add(problemBrowserWindow);
    //    this.addInternalFrame(problemBrowserFrame);
    //    problemBrowserFrame.pack();
    //    problemBrowserFrame.hide();

    methodBrowserPanel = new MethodBrowserPanel(this, loadMetHierarchy());
    methodBrowserWindow = new BrowserWindow(methodBrowserPanel);
    methodBrowserFrame = createInternalFrame("Method Browser", false, true,
                                             false, false);
    methodBrowserFrame.getContentPane().add(methodBrowserWindow);
    //    this.addInternalFrame(methodBrowserFrame);
    //    methodBrowserFrame.pack();
    //    methodBrowserFrame.hide();
  }

  /**
   * Open a new Worksheet and start the modeling and specification phase.
   * 
   * @param f
   *          Formalization (can be empty see UC 2.5.4) that the math engine
   *          needs to start a calculation
   * @param source
   *          Source of the Formalization (e.g. ExampleHierarchy or from
   *          Scratch)
   * @param type
   *          Requested ViewType for the CalcHead
   */
  public void openNewWorksheetWithCalchead(final Formalization f,
                                           final int source, final int type) {
    JInternalFrame internalFrame = createInternalFrame("Worksheet", true, true,
                                                       true, false);
    internalFrame.getContentPane().setLayout(new BorderLayout());
    final DGuide dialog = newDialog();

    final SwingWorker worker = new SwingWorker() {
      public Object construct() {
        Worksheet ws = new Worksheet(locale, dialog, f, source, type);
        return ws;
      }

      public void finished() {
        setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
      }
    };
    this.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
    worker.start();
    internalFrame.getContentPane().add((Worksheet) worker.get(),
                                       BorderLayout.CENTER);
    int width, height = 0;
    height = internalFrame.getPreferredSize().height;
    width = internalFrame.getPreferredSize().width;
    height = height < 300 ? 300 : height;
    width = width < 600 ? 600 : width;
    internalFrame.setPreferredSize(new Dimension(width, height));
    internalFrame.pack();
    internalFrame.setVisible(true);
    this.addInternalFrame(internalFrame);
  }

  public void openNewWorksheet(final int source, final int type) {
    final Formalization f = new Formalization();
    JInternalFrame internalFrame = createInternalFrame("Worksheet", true, true,
                                                       true, false);
    internalFrame.getContentPane().setLayout(new BorderLayout());
    final DGuide dialog = newDialog();

    final SwingWorker worker = new SwingWorker() {
      public Object construct() {
        Worksheet ws = new Worksheet(locale, dialog, f, source, type);
        return ws;
      }

      public void finished() {
        setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
      }
    };
    this.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
    worker.start();
    internalFrame.getContentPane().add((Worksheet) worker.get(),
                                       BorderLayout.CENTER);
    int width, height = 0;
    height = internalFrame.getPreferredSize().height;
    width = internalFrame.getPreferredSize().width;
    height = height < 300 ? 300 : height;
    width = width < 600 ? 600 : width;
    internalFrame.setPreferredSize(new Dimension(width, height));
    internalFrame.pack();
    internalFrame.setVisible(true);
    this.addInternalFrame(internalFrame);
  }

  //  public void openCalcHeadInWS(CalcHead ch) {
  //    if (activeFrame.getContentPane().getComponentCount() == 1) {
  //      CalcHeadPanel chp = new CalcHeadPanel(this, locale, ch, lastActiveWS);
  //      activeFrame.getContentPane().add(chp, BorderLayout.NORTH);
  //    } else if (activeFrame.getContentPane().getComponentCount() == 2) {
  //      CalcHeadPanel chp =
  // (CalcHeadPanel)activeFrame.getContentPane().getComponent(1);
  //      chp.setCalcHead(ch);
  //    }
  //    activeFrame.validate();
  //  }

  private JInternalFrame createInternalFrame(String title, boolean resizable,
                                             boolean closable,
                                             boolean maximizable,
                                             boolean iconifiable) {
    JInternalFrame internalFrame = new JInternalFrame(title, resizable,
        closable, maximizable, iconifiable);
    return internalFrame;
  }

  private void createWorksheetPane() {
    worksheetDesktopPane = new JDesktopPane();
    Dimension d = new Dimension();
    d.setSize(this.getWidth() * 0.60, 0.0);
    worksheetDesktopPane.setMinimumSize(d);
    worksheetDesktopPane.setDesktopManager(new DefaultDesktopManager() {
      public void activateFrame(JInternalFrame jif) {
        super.activateFrame(jif);
        activeFrame = jif;
      }
    });
    // "ghost frame" dragging; very fast but weired for some user
    worksheetDesktopPane.setDragMode(JDesktopPane.OUTLINE_DRAG_MODE);
  }

  /**
   * Get the currently active Worksheet (which has the cursor).
   * 
   * @return The active Worksheet.
   */
  public Worksheet getActiveWorksheet() {
    // assumes that the first component is the worksheet
    if (activeFrame != null) {
      Component comp = activeFrame.getContentPane().getComponent(0);
      if (comp instanceof Worksheet) {
        lastActiveWS = (Worksheet) comp;
      }
    }
    return lastActiveWS;
  }

  private void buildSplitPane() {
    splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, false);
    //splitPane.setLeftComponent(browserPanel);
    splitPane.setRightComponent(worksheetDesktopPane);
    splitPane.setOneTouchExpandable(true);
    splitPane.setDividerLocation(250);
  }

  private Point screenCenter() {
    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    int midX = screenSize.width / 2 - this.getWidth() / 2;
    int midY = screenSize.height / 2 - this.getHeight() / 2;
    return new Point(midX, midY);
  }

  private void addInternalFrame(JInternalFrame frame) {
    worksheetDesktopPane.add(frame);
    worksheetDesktopPane.getDesktopManager().activateFrame(frame);
    frame.toFront();
    frame.grabFocus();
  }

  /**
   * Get the session id of the current connection.
   * 
   * @return Systemwide unique session id
   */
  public String getSessionId() {
    return sessionId;
  }

  /**
   * Get the content Panel.
   * 
   * @return ContentPanel
   */
  public ContentPanel getContentPanel() {
    if (contentPanel == null) {
      this.contentPanel = new ContentPanel(sessionId);
    }
    return contentPanel;
  }

  /**
   * Get the currently used locale.
   * 
   * @return currently used locale.
   */
  public Locale getLocale() {
    return locale;
  }

  /**
   * Show the content frame which includes a window that shows the content of a
   * selected entry in the hierarchy.
   */
  public void showContentFrame() {
    if (contentFrame == null) {
      contentFrame = new JInternalFrame("Content Window", true, true, true);
      this.addInternalFrame(contentFrame);
      contentFrame.getContentPane().add(contentPanel, BorderLayout.CENTER);
      contentFrame.pack();
      contentFrame.setVisible(true);
    } else if (!contentFrame.isVisible()) {
      this.addInternalFrame(contentFrame);
      contentFrame.setVisible(true);
    }
  }

  //----------ActionListener----------------------------------------

  public void actionPerformed(ActionEvent e) {
    if (e.getSource() == problemBrowserButton) {
      if (!problemBrowserFrame.isVisible()) {
        problemBrowserFrame.setSize(300, 650);
        problemBrowserFrame.setLocation(700, 0);
        this.addInternalFrame(problemBrowserFrame);
        problemBrowserFrame.show();
      }
    } else if (e.getSource() == newWorksheetButton) {
      JOptionPane.showMessageDialog(this, "not yet implemented");
    } else if (e.getSource() == exampleBrowserButton) {
      if (!exampleBrowserFrame.isVisible()) {
        exampleBrowserFrame.setSize(300, 650);
        exampleBrowserFrame.setLocation(700, 0);
        this.addInternalFrame(exampleBrowserFrame);
        exampleBrowserFrame.show();
      }
    } else if (e.getSource() == methodBrowserButton) {
      if (!methodBrowserFrame.isVisible()) {
        methodBrowserFrame.setSize(300, 650);
        methodBrowserFrame.setLocation(700, 0);
        this.addInternalFrame(methodBrowserFrame);
        methodBrowserFrame.show();
      }
    } else if (e.getSource() == theoryBrowserButton) {
      JOptionPane.showMessageDialog(this, "not yet implemented");
    } else if (e.getSource() == nextStepButton) {
      getActiveWorksheet().nextStep();
    } else if (e.getSource() == autocalcButton) {
      getActiveWorksheet().autoCalc();
    } else if (e.getSource() == tacticsButton) {
      JOptionPane.showMessageDialog(this, "not yet implemented");
    } else { //invalid source
      logger.error("invalid source for ActionEvent");
    }
    //    Formalization f = new Formalization();
    //    openNewWorksheet(f, DialogGuide.STARTFROM_EMPTY,
    // CalcHead.VIEWSTYLE_SIMPLE_SOLVE);
  }

  public static void main(String[] args) {

    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        WindowApplication windowApplication = new WindowApplication();
        if (logger.isDebugEnabled())
          logger.debug("WindowApplication started");
      }
    });

  }

  /*
   * (non-Javadoc)
   * 
   * @see isac.gui.ILoginListener#loginEventOccured(isac.gui.LoginEvent)
   */
  public void loginEventOccured(LoginEvent loginEvent) {
    int type = loginEvent.getType();
    switch (type) {
    case (LoginEvent.LOGIN_TYPE_TRY_TO_LOGIN):
      String username = loginEvent.getUsername();
      String password = loginEvent.getPassword();
      this.login(username, password);
      break;
    case (LoginEvent.LOGIN_TYPE_EXIT):
      this.dispose();
      break;
    }
  }
}