/*
 * Created on Jul 5, 2003
 */
package isac.kestore;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.StringWriter;
import java.util.Vector;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
/**
 * @author Andreas Griesmayer
 */
public class KEStoreServices {
	private String path_;
	
	//WN040830 public for use in java-tests/../bridge
	public KEStoreServices(String path){
		path_=path;
		System.out.println("KEStoreServices default constructor called");
	}
	
	public Vector testMessage(){
		Vector result = new Vector();
		result.add("testMethod from KEStoreServices");
		return result;		
	}
	
	/**
	 *  Loads the hierarchy for type <code>type</code> from the filesystem.
	 * @param type
	 * @return Vector with the XML-representation of the hierarchy in its first item
	 */
	public Vector loadHierarchy(String type){
		Vector return_value=new Vector();
		try{
			DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
			DocumentBuilder db = dbf.newDocumentBuilder();
			Document document=null;
			System.out.println("type:"+type);
			if(type.equals("pbl"))
				document = db.parse(path_+"/pbl/pbl_hierarchy.xml");
			else if(type.equals("met"))
				document = db.parse(path_+"/met/met_hierarchy.xml");
			else if(type.equals("exp"))
				document = db.parse(path_+"/exp/exp__hierarchy.xml");
				
			DOMSource source = new DOMSource(document);
			StringWriter output = new StringWriter();
			StreamResult result = new StreamResult(output);
			
			TransformerFactory tFactory = TransformerFactory.newInstance();
			Transformer transformer = tFactory.newTransformer();
			transformer.transform(source, result);
//			XMLSerializer xmls = new XMLSerializer(output, new OutputFormat("xml", "utf-8", true));
//			xmls.serialize(document);
			System.out.println(output.toString());
			return_value.add(output.toString());		
		}
		catch(java.io.IOException exc) {
			System.out.println(exc.getMessage());
		}
		catch(Exception exc){
			System.out.println(exc.getMessage());
		}
		return return_value;
	}

/**
 *  loads the binary-file <code>file</code> for type <code>type</code>.
 * @param type type of the file to load - eg. exp
 * @param file to load - has to be in the directory of the type
 * @return a vector with a binary array of the file to load in its first element or empty array if the file was not found
 */
	public Vector loadBinary(String type, String file){
		System.out.println("binaryfile:"+file);
		Vector return_value = new Vector();
		File ifile = new File(path_+"/"+type+"/"+file);
		try{		
			FileInputStream fis = new FileInputStream(ifile);
			int size = fis.available();
			byte[] data=new byte[size];
			fis.read(data);
			return_value.add(data);
			
		}catch(FileNotFoundException exc){
			return_value.removeAllElements();
			return_value.add(new byte[0]);
			return return_value;
		} catch (IOException e) {
			return_value.removeAllElements();
			return_value.add(new byte[0]);
			return return_value;
		}
		return return_value;
	}

	public Vector loadContent(String type, Vector id){
		Vector return_value=new Vector();
		try{
			DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
			DocumentBuilder db = dbf.newDocumentBuilder();
			Document document=null;
			System.out.println("type:"+type);
			document = db.parse(path_+"/pbl/pbl_hierarchy.xml");
				
			DOMSource source = new DOMSource(document);

			boolean	found=false;
			
			Node basenode = source.getNode();
			Node pblnode = basenode.getChildNodes().item(0);
			while(id.size()>0){
				NodeList childlist = pblnode.getChildNodes();
				String id_part = (String)id.remove(0);
				for(int i=0;i < childlist.getLength(); i++){
					if(!childlist.item(i).getNodeName().trim().equals("NODE"))
						continue;	
					if(childlist.item(i).getChildNodes().item(1).getFirstChild().getNodeValue().trim().equals(id_part)){
						pblnode=childlist.item(i);
						found=true;
						break;
					}
				}
			}
			
			if(!found)
				return new Vector();	
			
			String filename="";
			if(pblnode instanceof Element){
				NodeList cnt_lst=((Element)pblnode).getElementsByTagName("*");
				for(int i=0;i<cnt_lst.getLength();i++){
					if(cnt_lst.item(i).getNodeName().trim().equals("CONTENTREF")){					
						filename=cnt_lst.item(i).getChildNodes().item(0).getNodeValue().trim();
						break;
					}
				}
			}
			Vector contentvect = loadContent(type, filename);
			contentvect.add(filename); 
			return contentvect;
		}
		catch(java.io.IOException exc) {
			System.out.println(exc.getMessage());
		}
		catch(Exception exc){
			System.out.println(exc.getMessage());
		}
	 	return new Vector();
	}

/**
 * loads an KE-Object in its XML-representation. uses an XML-DocumentBuilder to validate if the file is in proper format
 * future implementation shall create the XML-representations out of smaller chunks - a final data-format is not found yet.
 * 
 * @param type of the KE-Object (like a collection)
 * @param file id of the KE-Object to load (filename in the current implementation)
 * @return Vector with the KE-Object in its XML-representation in its first element or empty Vector if the file was not found.
 */
	public Vector loadContent(String type, String file){
		System.out.println("File:"+file);
		Vector return_value=new Vector();
		try{
			DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
			DocumentBuilder db = dbf.newDocumentBuilder();
			Document document = db.parse(path_+"/"+type+"/"+file);
			DOMSource source = new DOMSource(document);
			StringWriter output = new StringWriter();
			StreamResult result = new StreamResult(output);
		
			TransformerFactory tFactory = TransformerFactory.newInstance();
			Transformer transformer = tFactory.newTransformer();
			transformer.transform(source, result);
			return_value.add(output.toString()); //WN040830 why Vector for 1 element ?
		}catch(Exception exc){
			System.out.println("KEStore:Exception "+exc.getMessage());
		}
		return return_value;
	}
/*		
	public Vector getXMLFile(String file){
	Vector return_value=new Vector();
		try{
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		Document document = db.parse(file);	
		StringWriter output = new StringWriter();
		
	 	XMLSerializer xmls = new XMLSerializer(output, 
													   new OutputFormat("xml","utf-8",true)); 
		try { 
			xmls.serialize(document);
		}
		catch(java.io.IOException exc) {
			System.out.println(exc.getMessage());
		}
		return_value.add(output.toString());
		
		}catch(Exception exc){
			System.out.println(exc.getMessage());
		}
		return return_value;
	}
	*/
}


