/****************************************************************** 
* Copyright (c) 2003, Mario Hochreiter, Student of 
* Software Engineering at FH-Hagenberg, Austria. 
* Project member of the isac team at the Institute for 
* Software-Technologie, Graz University of Technology, Austria. 
* 
* Use is subject to license terms.
******************************************************************/

package isac.util;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;

/**
 * Every calculation in the isac system starts with a formalization 
 * that characterizes the calculation. A formalization object can 
 * contain multiple <code>Variant</code> objects because a calculation
 * can be calculated in different ways (using different methods to solve
 * the calculation).
 *
 * @author Mario Hochreiter
 * @version 0.1
 *
 */
public class Formalization implements Serializable {
  protected ArrayList variants;

  public Formalization() {
    variants = new ArrayList();
  }

  public void addNewVariant(Variant v) {
    variants.add(v);
  }

  public Iterator iterator() {
    return new FormalizationIterator(this);
  }

	public String toString(){
		String content="<FORMALIZATION>\n";
	 for(int index=0; index<variants.size(); index++)
	 	content += variants.get(index)+"\n";
	 content += "</FORMALIZATION>\n";
	 return content;
	}

  /** Format the object in a way that the bridge can handle it 
   *  @return SML representation of this Formalization
   */
  public String toSMLString() {
  	// Use this String for empty Formalization: 
  	// Calculation from scratch
		
	if (variants.isEmpty()) return "[([], (\"e_domID\", [\"e_pblID\"], [\"e_metID\"]))]";
	// e_domID, etc. will disappear soon: empty formalization would be "[([], (\"\", [], []))]"
    StringBuffer sb = new StringBuffer();    
    int size = variants.size();
    int i = 0;
    Variant v;
    
    sb.append("[");
    Iterator it = variants.iterator();
    while (it.hasNext()) {
      v = (Variant)it.next();
      sb.append("(");
      sb.append(v.toSMLString()); 
      sb.append(")");
      if (i < size - 1) {
        sb.append(",");
      }
      i++;
    }
    sb.append("]");    
    return sb.toString();
	 
  }
}

class FormalizationIterator implements Iterator {
  Iterator it;

  FormalizationIterator(Formalization f) {
    it = f.variants.iterator();
  }

  public void remove() {
    it.remove();
  }

  public boolean hasNext() {
    return it.hasNext();
  }

  public Object next() {
    return it.next();
  }
}
