/****************************************************************** 
* Copyright (c) 2003, Mario Hochreiter, Student of 
* Software Engineering at FH-Hagenberg, Austria. 
* Project member of the isac team at the Institute for 
* Software-Technologie, Graz University of Technology, Austria. 
* 
* Use is subject to license terms.
******************************************************************/

package isac.util;

import isac.util.formulae.Specification;

import java.io.Serializable;
import java.util.ArrayList;

/**
 * Describes a way how a specific calculation will be calculated (which
 * method will be used to solve this calculation). 
 * 
 * @author Mario Hochreiter
 * @version 0.1
 *
 */
public class Variant implements Serializable {
  /** strings betweend the isa tags */
  protected ArrayList isaStrings;
  
  protected Specification spec;
  
  public Variant() {
    isaStrings = new ArrayList();
  }
  
  /** 
   *  Add a new isa string 
   *  @param isa The isa string
   */
  public void addNewIsaString(String isa) {
    isaStrings.add(isa);
  }
  
  public String toString(){
  	String content="<TERMLIST>";
  	for(int index=0;index<isaStrings.size();index++)
  		content+="<MATHML>\n<ISA>\n"+isaStrings.get(index)+"\n</ISA>\n</MATHML>\n";
  		
  	if(spec!=null){
		content+=spec;
  	}
  	content+="</TERMLIST>\n";
  	return content;
  }
  
  /** Format the object in a way that the bridge can handle it 
   *  @return SML representation of this Variant
   */
  public String toSMLString() {
    StringBuffer sb = new StringBuffer();
    
    int size = isaStrings.size();
    sb.append("[");
    for (int i = 0; i < size; i++) {
      sb.append("\""+(String)isaStrings.get(i)+"\"");
      if (i < size - 1)
        sb.append(", ");
    }
    sb.append("],");
	  sb.append("("+spec.toSMLString()+")");
    return sb.toString();
  }
  
  public void setSpecification(Specification spec) {
    this.spec = spec;
  }
  
  public Specification getSpecification() {
    return spec;
  }
  
}
